from logging import Logger
from typing import List
from penvy.tear_down.TearDownStepInterface import TearDownStepInterface


class TearDownRunner:
    def __init__(
        self,
        steps: List[TearDownStepInterface],
        logger: Logger,
    ):
        self._steps = steps
        self._logger = logger

    def run(self):
        for step in self._steps:
            self._logger.debug(f"Running {step.__module__}")

            step.run()



import os
from argparse import Namespace
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface
from penvy.conda.CondaExecutablePathFinder import CondaExecutablePathFinder
from penvy.shell.runner import run_and_read_line


class CondaPathsResolver(ParametersResolverInterface):
    def __init__(
        self,
        conda_executable_path_finder: CondaExecutablePathFinder,
    ):
        self._conda_executable_path_finder = conda_executable_path_finder

    def resolve(self, config: dict, cli_args: Namespace):
        conda_executable_path = self._conda_executable_path_finder.find()
        conda_install_dir = run_and_read_line(f"{conda_executable_path} info --base", shell=True)

        return {
            "project": {
                "dir": os.getcwd(),
                "venv_dir": os.getcwd() + os.sep + ".venv",
            },
            "conda": {
                "executable_path": conda_executable_path,
                "install_dir": conda_install_dir,
                "conda_sh_path": conda_install_dir + "/etc/profile.d/conda.sh",
            },
        }



from argparse import Namespace
from typing import List
from penvy.parameters.ParametersMerger import ParametersMerger
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface


class ResolvedParametersBuilder:
    def __init__(self, resolvers: List[ParametersResolverInterface]):
        self._resolvers = resolvers
        self._config_merger = ParametersMerger()

    def build(self, cli_args: Namespace):
        config = dict()

        for resolver in self._resolvers:
            new_config = resolver.resolve(config, cli_args)

            self._config_merger.merge(config, new_config)

        return config



import os
import shutil
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface


class DotEnvCreator(SetupStepInterface):
    def __init__(
        self,
        project_dir: str,
        logger: Logger,
    ):
        self._logger = logger
        self._dot_env_path = project_dir + os.sep + ".env"
        self._dot_env_dist_path = project_dir + os.sep + ".env.dist"

    def get_description(self):
        return "Create .env file from the .env.dist template"

    def run(self):
        self._logger.info("Creating .env file from the .env.dist template")

        shutil.copy(self._dot_env_dist_path, self._dot_env_path)

    def should_be_run(self) -> bool:
        return os.path.isfile(self._dot_env_dist_path) and not os.path.isfile(self._dot_env_path)



from abc import ABC, abstractmethod


class SetupStepInterface(ABC):
    @abstractmethod
    def get_description(self):
        pass

    @abstractmethod
    def run(self):
        pass

    @abstractmethod
    def should_be_run(self) -> bool:
        pass



from argparse import ArgumentParser
from penvy.cli.str2bool import str2bool


def create_argument_parser():
    argument_parser = ArgumentParser()
    argument_parser.add_argument(
        "-v", "--verbose", dest="verbose", type=str2bool, nargs="?", const=True, default=False, help="Enable verbose logging"
    )
    argument_parser.add_argument(
        "-y", "--yes", dest="skip_confirmation", type=str2bool, nargs="?", const=True, default=False, help="Skip confirmation"
    )

    return argument_parser



def diservice(method):
    def wrapper(*args):
        service_name = method.__name__
        container = args[0]

        if service_name not in container.services:
            # print('creating service ' + service_name)
            container.services[service_name] = method(*args)

        return container.services[service_name]

    return wrapper



import filecmp
import os
import platform
import subprocess
from shutil import copyfile
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.libroot import get_libroot


class CondaScriptsCreator(SetupStepInterface):
    def __init__(
        self,
        venv_dir: str,
        logger: Logger,
    ):
        self._venv_dir = venv_dir
        self._activate_script_source_path = f"{get_libroot()}/conda/activate.d/env_vars.sh"
        self._activate_script_target_path = f"{venv_dir}/etc/conda/activate.d/env_vars.sh"
        self._deactivate_script_source_path = f"{get_libroot()}/conda/deactivate.d/env_vars.sh"
        self._deactivate_script_target_path = f"{venv_dir}/etc/conda/deactivate.d/env_vars.sh"
        self._logger = logger

    def get_description(self):
        return "Create/update conda environment activation and deactivation scripts"

    def run(self):
        self._logger.info("Setting up Conda activation & deactivation scripts")

        self._logger.info("Seting-up conda/activate.d")
        self._copy_script(self._activate_script_source_path, self._activate_script_target_path)

        self._logger.info("Seting-up conda/deactivate.d")
        self._copy_script(self._deactivate_script_source_path, self._deactivate_script_target_path)

    def _copy_script(self, script_source_path: str, script_target_path: str):
        os.makedirs(os.path.dirname(script_target_path), exist_ok=True)

        copyfile(script_source_path, script_target_path)

        if platform.system() != "Windows":
            subprocess.check_call(["chmod", "+x", script_target_path])

    def should_be_run(self) -> bool:
        return (
            not os.path.exists(self._activate_script_target_path)
            or not os.path.exists(self._deactivate_script_target_path)
            or not filecmp.cmp(self._activate_script_source_path, self._activate_script_target_path)
            or not filecmp.cmp(self._deactivate_script_source_path, self._deactivate_script_target_path)
        )



import os
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.shell.runner import run_shell_command


class CondaEnvironmentCreator(SetupStepInterface):
    def __init__(
        self,
        conda_executable_path: str,
        venv_dir: str,
        logger: Logger,
    ):
        self._conda_executable_path = conda_executable_path
        self._venv_dir = venv_dir
        self._logger = logger

    def get_description(self):
        return "Create the project conda environment"

    def run(self):
        self._logger.info(f"Creating Conda environment in {self._venv_dir}")

        run_shell_command(f"{self._conda_executable_path} env create -f environment.yml -p {self._venv_dir}", shell=True)

    def should_be_run(self) -> bool:
        return not os.path.isdir(self._venv_dir)



from argparse import Namespace
from typing import Optional
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface
from penvy.shell.detector import detect_shell, ShellVarNotSetException, BASH, GIT_BASH, ZSH


class ShellResolver(ParametersResolverInterface):
    def resolve(self, config: dict, cli_args: Namespace):
        try:
            shell = detect_shell()
        except ShellVarNotSetException:
            return self._return(None, False, False)

        is_bash = shell in [BASH, GIT_BASH]
        is_zsh = shell == ZSH

        return self._return(shell, is_bash, is_zsh)

    def _return(self, name: Optional[str], is_bash: bool, is_zsh: bool):
        return {
            "shell": {
                "name": name,
                "is_bash": is_bash,
                "is_zsh": is_zsh,
            },
        }



import os
import shutil


class CondaExecutablePathFinder:
    def find(self):
        homedir = os.path.expanduser("~")

        if shutil.which("conda"):
            return shutil.which("conda")

        conda_paths = {
            f"{homedir}/Miniconda3/Library/bin/conda.bat": f"{homedir}/Miniconda3",
            f"{homedir}/Anaconda3/Library/bin/conda.bat": f"{homedir}/Anaconda3",
            f"{homedir}/AppData/Local/Continuum/miniconda3/condabin/conda.bat": f"{homedir}/AppData/Local/Continuum/miniconda3",
            f"{homedir}/AppData/Local/Continuum/anaconda3/condabin/conda.bat": f"{homedir}/AppData/Local/Continuum/anaconda3",
            f"{homedir}/miniconda3/bin/conda": f"{homedir}/miniconda3",
            f"{homedir}/anaconda3/bin/conda": f"{homedir}/anaconda3",
            f"{homedir}/miniconda/bin/conda": f"{homedir}/miniconda",
            f"{homedir}/anaconda/bin/conda": f"{homedir}/anaconda",
        }

        for conda_executable_path, conda_installation_dir in conda_paths.items():
            if os.path.isfile(conda_executable_path):
                return conda_executable_path

        raise Exception("Unable to find Conda executable, exiting...")



import os
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.string.string_in_file import file_contains_string


class DependenciesInstallHookCreator(SetupStepInterface):
    def __init__(
        self,
        post_merge_hook_path: str,
        poetry_executable_path: str,
        logger: Logger,
    ):
        self._post_merge_hook_path = post_merge_hook_path
        self._poetry_executable_path = poetry_executable_path
        self._logger = logger

    def get_description(self):
        return "Create git-hook to automatically install new project dependencies after git pull/merge"

    def run(self):
        self._logger.info("Adding poetry install to post-merge git hook")

        with open(self._post_merge_hook_path, "a", encoding="utf-8") as f:
            f.write(f"{self._poetry_executable_path} install --no-root\n")

    def should_be_run(self) -> bool:
        return not os.path.isfile(self._post_merge_hook_path) or not file_contains_string(
            "poetry install --no-root", self._post_merge_hook_path
        )



import re
from distutils.version import StrictVersion
from logging import Logger
from penvy.check.CheckInterface import CheckInterface
from penvy.shell.runner import run_and_read_line


class GitVersionCheck(CheckInterface):
    def __init__(
        self,
        minimal_version: str,
        logger: Logger,
    ):
        self._minimal_version = minimal_version
        self._logger = logger

    def run(self):
        first_line = run_and_read_line("git --version", shell=True)

        git_version = re.sub(r"^git version ([0-9.]+)[.].+$", "\\1", first_line)

        if StrictVersion(git_version) < StrictVersion(self._minimal_version):
            return f"Conda version {git_version} is too old, please update to {self._minimal_version} or higher"

        self._logger.debug(f"Git version {git_version} ok")



from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.shell.runner import run_shell_command


class DependenciesInstaller(SetupStepInterface):
    def __init__(
        self,
        poetry_executable_path: str,
        logger: Logger,
    ):
        self._poetry_executable_path = poetry_executable_path
        self._logger = logger

    def get_description(self):
        return "Install all python dependencies"

    def run(self):
        self._logger.info("Installing dependencies from poetry.lock")

        run_shell_command(f"{self._poetry_executable_path} install --no-root", shell=True)

    def should_be_run(self) -> bool:
        return True



import sys
from logging import Logger
from penvy.check.CheckInterface import CheckInterface
from penvy.shell.detector import detect_shell, ShellVarNotSetException, WIN_CMD, ZSH, BASH, GIT_BASH


class ShellCheck(CheckInterface):
    def __init__(
        self,
        logger: Logger,
    ):
        self._logger = logger

    def run(self):
        try:
            shell_name = detect_shell()
        except ShellVarNotSetException:
            self._logger.warning("Unable to resolve shell type from the SHELL env variable")
            return

        if shell_name == WIN_CMD and sys.argv[0][-8:] != "/init.py":
            return "Cannot run under cmd.exe, please use Git Bash instead"

        if shell_name not in [ZSH, BASH, GIT_BASH]:
            self._logger.warning(f"Your shell {shell_name} is not fully supported, use bash or zsh instead")
            return

        self._logger.debug(f"Shell {shell_name} ok")



import os
from argparse import Namespace

from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface


class PythonExecutablePathResolver(ParametersResolverInterface):
    def resolve(self, config: dict, cli_args: Namespace):
        venv_dir = config["project"]["venv_dir"]

        if os.name == "nt":
            python_executable_path = venv_dir + "\\python.exe"
        else:
            python_executable_path = venv_dir + "/bin/python"

        return {
            "python": {
                "executable_path": python_executable_path,
            }
        }



from logging import Logger
from penvy.tear_down.TearDownStepInterface import TearDownStepInterface


class ActivateCondaEnvCall(TearDownStepInterface):
    def __init__(
        self,
        venv_dir: str,
        logger: Logger,
    ):
        self._venv_dir = venv_dir
        self._logger = logger

    def run(self):
        venv_path_forward_slashes = self._venv_dir.replace("\\", "/")

        self._logger.info("Setup completed. Active Conda environment now:")
        self._logger.info(f"conda activate {venv_path_forward_slashes}")



from penvy.PenvyConfig import PenvyConfig
from penvy.container.dicontainer import Container
from penvy.env.EnvInitRunner import EnvInitRunner


def main():
    runner = EnvInitRunner([PenvyConfig()], Container)
    runner.run()


if __name__ == "__main__":
    main()



import sys
from typing import List
from penvy.PenvyConfig import PenvyConfig
from penvy.cli.argument_parser import create_argument_parser
from penvy.container.init import resolve_parameters, create_checks_runner, create_setup_runner, create_tear_down_runner
from penvy.poetry.DependenciesInstaller import DependenciesInstaller
from penvy.setup.SetupStepInterface import SetupStepInterface


class EnvInitRunner:
    def __init__(self, env_configs: List[PenvyConfig], container_class):
        argument_parser = self._create_argument_parser()
        self._cli_arguments = argument_parser.parse_known_args()[0]
        self._env_configs = env_configs
        self._container = container_class(self._resolve_parameters())
        self._logger = self._container.get_logger()
        self._checks_runner = self._create_checks_runner()
        self._setup_runner = self._create_setup_runner()
        self._tear_down_steps_runner = self._get_tear_down_steps_runner()

    def run(self):
        steps2run = self._setup_runner.get_steps2run()

        if self._skip_confirmation(steps2run):
            self._checks_runner.run()
        else:
            errors = self._checks_runner.check()

            if errors:
                self._checks_runner.print_errors(errors, "Fix the following errors and try again:")
                self._setup_runner.show(steps2run)
                sys.exit(1)

            self._setup_runner.show(steps2run)

            should_continue = input("\nContinue? [y/n]: ")

            if should_continue not in ["", "y", "n"]:
                self._logger.error('Press Enter, "y" or "n"')
                sys.exit(1)
            elif should_continue == "n":
                sys.exit(0)

        self._setup_runner.run()
        self._tear_down_steps_runner.run()

    def _skip_confirmation(self, steps2run: List[SetupStepInterface]):
        return self._cli_arguments.skip_confirmation or (len(steps2run) == 1 and isinstance(steps2run[0], DependenciesInstaller))

    def _create_argument_parser(self):
        return create_argument_parser()

    def _resolve_parameters(self):
        return resolve_parameters(self._env_configs, self._cli_arguments)

    def _create_checks_runner(self):
        return create_checks_runner(self._env_configs, self._container)

    def _create_setup_runner(self):
        return create_setup_runner(self._env_configs, self._container)

    def _get_tear_down_steps_runner(self):
        return create_tear_down_runner(self._env_configs, self._container)



from argparse import Namespace
from typing import List
from penvy.env.EnvConfig import EnvConfig
from penvy.check.CheckInterface import CheckInterface
from penvy.parameters.ParametersMerger import ParametersMerger
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface
from penvy.parameters.ResolvedParametersBuilder import ResolvedParametersBuilder
from penvy.container.dicontainer import Container
from penvy.setup.SetupRunner import SetupRunner
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.check.ChecksRunner import ChecksRunner
from penvy.tear_down.TearDownRunner import TearDownRunner
from penvy.tear_down.TearDownStepInterface import TearDownStepInterface


def create_checks_runner(env_configs: List[EnvConfig], container: Container):
    checks: List[CheckInterface] = []

    for env_config in env_configs:
        checks = checks + env_config.get_checks(container)

    return ChecksRunner(checks, container.get_logger())


def create_setup_runner(env_configs: List[EnvConfig], container: Container):
    setup_steps: List[SetupStepInterface] = []

    for env_config in env_configs:
        setup_steps = setup_steps + env_config.get_setup_steps(container)

    return SetupRunner(setup_steps, container.get_logger())


def create_tear_down_runner(env_configs: List[EnvConfig], container: Container):
    tear_down_steps: List[TearDownStepInterface] = []

    for env_config in env_configs:
        tear_down_steps = tear_down_steps + env_config.get_tear_down_steps(container)

    return TearDownRunner(tear_down_steps, container.get_logger())


def resolve_parameters(env_configs: List[EnvConfig], cli_args: Namespace):
    parameters_merger = ParametersMerger()

    parameters = {}
    config_resolvers: List[ParametersResolverInterface] = []

    for env_config in env_configs:
        parameters = parameters_merger.merge(parameters, env_config.get_parameters())

    for env_config in env_configs:
        config_resolvers = config_resolvers + env_config.get_parameters_resolvers(parameters)

    resolved_parameters_builder = ResolvedParametersBuilder(config_resolvers)
    resolved_parameters = resolved_parameters_builder.build(cli_args)

    return parameters_merger.merge(parameters, resolved_parameters)



import os
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface


class BashProfileCreator(SetupStepInterface):
    def __init__(
        self,
        is_running_bash: bool,
        logger: Logger,
    ):
        self._is_running_bash = is_running_bash
        self._logger = logger
        self._dot_bash_profile_path = os.path.expanduser("~") + os.sep + ".bash_profile"

    def get_description(self):
        return "Create the default .bash_profile file"

    def run(self):
        self._logger.info(f"Creating default {self._dot_bash_profile_path}")

        with open(self._dot_bash_profile_path, "w", encoding="utf-8") as f:
            f.write("test -f ~/.profile && . ~/.profile\n")
            f.write("test -f ~/.bashrc && . ~/.bashrc\n")

    def should_be_run(self) -> bool:
        return self._is_running_bash and not os.path.isfile(self._dot_bash_profile_path)



import os
import shutil
import urllib.request
import tempfile
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.shell.runner import run_shell_command
from penvy.string.random_string_generator import generate_random_string


class PoetryInstaller(SetupStepInterface):
    def __init__(
        self,
        python_executable_path: str,
        poetry_executable_path: str,
        install_version: str,
        logger: Logger,
    ):
        self._python_executable_path = python_executable_path
        self._poetry_executable_path = poetry_executable_path
        self._install_version = install_version
        self._logger = logger

    def get_description(self):
        return f"Install poetry {self._install_version}"

    def run(self):
        self._logger.info("Installing Poetry globally")

        tmp_dir = tempfile.gettempdir()
        target_file_name = tmp_dir + f"/get-poetry_{generate_random_string(5)}.py"

        url = "https://raw.githubusercontent.com/sdispater/poetry/master/get-poetry.py"
        urllib.request.urlretrieve(url, target_file_name)

        run_shell_command(f"{self._python_executable_path} {target_file_name} -y --version {self._install_version}", shell=True)

    def should_be_run(self) -> bool:
        return shutil.which("poetry") is None and not os.path.isfile(self._poetry_executable_path)



import sys
from logging import Logger
from typing import List
from penvy.check.CheckInterface import CheckInterface


class ChecksRunner:
    def __init__(
        self,
        checks: List[CheckInterface],
        logger: Logger,
    ):
        self._checks = checks
        self._logger = logger

    def run(self):
        errors = self.check()

        if errors:
            self.print_errors(errors, "Following errors found:")
            sys.exit(1)

    def check(self):
        errors = []

        for check in self._checks:
            check_result = check.run()

            if check_result:
                errors.append(check_result)

        return errors

    def print_errors(self, errors: list, message: str):
        self._logger.error(message)

        for index, error in enumerate(errors):
            self._logger.error(f"{index + 1}. {error}")



from abc import ABC
from typing import List
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface
from penvy.container.dicontainer import Container


class EnvConfig(ABC):
    def get_parameters(self) -> dict:
        return {}

    def get_parameters_resolvers(self, default_config: dict) -> List[ParametersResolverInterface]:
        return []

    def get_checks(self, container: Container):
        return []

    def get_setup_steps(self, container: Container):
        return []

    def get_tear_down_steps(self, container: Container):
        return []



import logging
from argparse import Namespace
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface


class LoggerLevelResolver(ParametersResolverInterface):
    def resolve(self, config: dict, cli_args: Namespace):
        return {
            "logger": {"level": logging.DEBUG if cli_args.verbose is True else logging.INFO},
        }



from logging import Logger, DEBUG
from typing import List
from penvy.setup.SetupStepInterface import SetupStepInterface


class SetupRunner:
    def __init__(
        self,
        steps: List[SetupStepInterface],
        logger: Logger,
    ):
        self._steps = steps
        self._logger = logger

    def get_steps2run(self):
        return list(filter(lambda step: step.should_be_run(), self._steps))

    def show(self, steps2run: List[SetupStepInterface]):
        self._logger.info("Setup steps to run:")

        is_verbose = self._logger.level == DEBUG
        index = 1

        for step in steps2run:
            description = self._get_description(step, is_verbose)
            self._logger.info(f"{index}. {description}")
            index += 1

    def run(self):
        is_verbose = self._logger.level == DEBUG

        for step in self._get_steps2skip():
            description = self._get_description(step, is_verbose)
            self._logger.debug(f"Skipping: {description}")

        for step in self.get_steps2run():
            description = self._get_description(step, is_verbose)
            self._logger.debug(f"Running: {description}")
            step.run()

    def _get_steps2skip(self):
        return list(filter(lambda step: not step.should_be_run(), self._steps))

    def _get_description(self, step: SetupStepInterface, is_verbose):
        return f"{step.get_description()}" + (f" ({step.__module__})" if is_verbose else "")



import os
import platform
import subprocess
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface


class PostMergeHookCreator(SetupStepInterface):
    def __init__(
        self,
        post_merge_hook_path: str,
        logger: Logger,
    ):
        self._post_merge_hook_path = post_merge_hook_path
        self._logger = logger

    def get_description(self):
        return "Create empty post-merge git hook file"

    def run(self):
        self._logger.info("Creating empty post-merge hook file")

        with open(self._post_merge_hook_path, "w", encoding="utf-8") as f:
            f.write("#!/bin/sh\n\n")

        if platform.system() != "Windows":
            subprocess.check_call(["chmod", "+x", self._post_merge_hook_path])

    def should_be_run(self) -> bool:
        return not os.path.isfile(self._post_merge_hook_path)



import os
import shutil
import stat


def rmdir(dir_to_delete: str):
    def on_rm_error(func, path, exc_info):
        # path contains the path of the file that couldn't be removed
        # let's just assume that it's read-only and unlink it.
        os.chmod(path, stat.S_IWRITE)
        os.unlink(path)

    shutil.rmtree(dir_to_delete, onerror=on_rm_error)



import io
import os
import subprocess
import sys


def _log_subprocess_output(pipe):
    for line in iter(pipe.readline, b""):  # b'\n'-separated lines
        print(line.decode("utf-8").replace("\r\n", ""))


def run_and_read_line(command: str, shell=False):
    proc = subprocess.Popen(command, stdout=subprocess.PIPE, shell=shell)
    return io.TextIOWrapper(proc.stdout, encoding="utf-8").read().replace("\n", "")


def run_shell_command(command: str, base_path: str = os.getcwd(), shell=False):
    proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=base_path, shell=shell)
    with proc.stdout:
        _log_subprocess_output(proc.stdout)

    exitcode = proc.wait()  # 0 means success

    if exitcode > 0:
        sys.exit(exitcode)



import os
from argparse import Namespace
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface


class PoetryPathResolver(ParametersResolverInterface):
    def resolve(self, config: dict, cli_args: Namespace):
        return {
            "poetry": {
                "executable_path": os.path.expanduser("~") + "/.poetry/bin/poetry",
            },
        }



"""A logging formatter for colored output."""

from __future__ import absolute_import

from penvy.logger.colorlog.colorlog import escape_codes, default_log_colors, ColoredFormatter, LevelFormatter, TTYColoredFormatter

from penvy.logger.colorlog.logging import basicConfig, root, getLogger, log, debug, info, warning, error, exception, critical, StreamHandler

__all__ = (
    "ColoredFormatter",
    "default_log_colors",
    "escape_codes",
    "basicConfig",
    "root",
    "getLogger",
    "debug",
    "info",
    "warning",
    "error",
    "exception",
    "critical",
    "log",
    "exception",
    "StreamHandler",
    "LevelFormatter",
    "TTYColoredFormatter",
)



import os
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface


class PyfonyEnvCreator(SetupStepInterface):
    def __init__(
        self,
        logger: Logger,
    ):
        self._logger = logger
        self._pyfony_env_script_path = os.path.expanduser("~") + os.sep + "pyfony_env.sh"

    def get_description(self):
        return "Create the pyfony_env.sh containing the ca alias to conda activate"

    def run(self):
        self._logger.info("Creating pyfony_env.sh")

        with open(self._pyfony_env_script_path, "w", encoding="utf-8") as f:
            f.write("alias ca='conda activate $PWD/.venv'")

    def should_be_run(self) -> bool:
        return not os.path.exists(self._pyfony_env_script_path)



import os
from logging import Logger
from penvy.check.CheckInterface import CheckInterface


class PyprojectChecker(CheckInterface):
    def __init__(
        self,
        logger: Logger,
    ):
        self._logger = logger
        self._pyproject_file_path = os.getcwd() + os.sep + "pyproject.toml"

    def run(self):
        if not os.path.isfile(self._pyproject_file_path):
            return "pyproject.toml file required by Poetry is missing in the current folder"

        self._logger.debug("pyproject.toml ok")



import penvy.logger.colorlog as colorlog


def get_logger(name: str, level: int):
    logger = colorlog.getLogger(name)
    logger.setLevel(level)

    cformat = "%(log_color)s" + "%(asctime)s - %(message)s"
    formatter = colorlog.ColoredFormatter(cformat, "%H:%M:%S")

    ch = colorlog.StreamHandler()
    ch.setFormatter(formatter)
    ch.setLevel(level)

    logger.handlers = [ch]

    return logger



import os
import platform
from penvy.cmd.detector import is_windows_cmd
from penvy.shell.name_extractor import extract_shell_name

WIN_CMD = "win_cmd"
GIT_BASH = "git_bash"
BASH = "bash"
ZSH = "zsh"


class ShellVarNotSetException(Exception):
    pass


def detect_shell():
    if is_windows_cmd():
        return WIN_CMD

    if platform.system() == "Windows":
        return GIT_BASH

    if "ZSH_VERSION" in os.environ:
        return ZSH

    if "BASH_VERSION" in os.environ:
        return BASH

    if "SHELL" not in os.environ:
        raise ShellVarNotSetException()

    return extract_shell_name(os.environ["SHELL"])



import os
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.shell.home_path_shortener import shorten_home_path
from penvy.string.string_in_file import file_contains_string


class SourceCondaAppender(SetupStepInterface):
    def __init__(
        self,
        rc_file_path,
        conda_sh_path: str,
        expected_shell_is_running: callable,
        logger: Logger,
    ):
        self._rc_file_path = rc_file_path
        self._rc_file_path_shorten = shorten_home_path(rc_file_path)
        self._conda_sh_path = conda_sh_path
        self._conda_sh_path_shorten = shorten_home_path(conda_sh_path)
        self._expected_shell_is_running = expected_shell_is_running
        self._logger = logger

    def get_description(self):
        return f'Add "source {self._conda_sh_path_shorten}" into {self._rc_file_path_shorten}'

    def run(self):
        conda_sh_path_foward_slashes = self._conda_sh_path_shorten.replace("\\", "")

        self._logger.info(f"Adding {conda_sh_path_foward_slashes} to {self._rc_file_path_shorten}")

        with open(self._rc_file_path, "a") as f:
            f.write(f"source {conda_sh_path_foward_slashes}\n")

    def should_be_run(self) -> bool:
        return self._expected_shell_is_running and (
            not os.path.isfile(self._rc_file_path) or not file_contains_string("/etc/profile.d/conda.sh", self._rc_file_path)
        )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['penvy', 'penvy.logger.colorlog']

package_data = \
{'': ['*'],
 'penvy': ['bash/*',
           'check/*',
           'cli/*',
           'cmd/*',
           'conda/*',
           'conda/activate.d/*',
           'conda/deactivate.d/*',
           'container/*',
           'dotenv/*',
           'env/*',
           'filesystem/*',
           'git/*',
           'logger/*',
           'parameters/*',
           'poetry/*',
           'python/*',
           'setup/*',
           'shell/*',
           'string/*',
           'tear_down/*']}

entry_points = \
{'console_scripts': ['penvy-init = penvy.init:main']}

setup_kwargs = {
    'name': 'penvy',
    'version': '1.0.0a2',
    'description': 'Pyfony framework development environment initializer',
    'long_description': "# Penvy - Pyfony development environment\n\nfor the [Pyfony framework](https://github.com/pyfony/pyfony)\n\n### What it does:\n\n* Prepares the **Conda-based python dev environment** in the project's **.venv directory**\n* Installs the [Poetry package manager](https://python-poetry.org/) into the user's home dir\n* Installs all the dependencies defined in project's **poetry.lock** file\n* Sets conda activation & deactivation scripts (mostly setting environment variables based on the project's **.env file**)\n* Copies the project's **.env file** from the **.env.dist** template file\n* Adds `poetry install --no-root` to post-merge GIT hook \n",
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pyfony/penvy',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import List
from penvy.env.EnvConfig import EnvConfig
from penvy.conda.CondaExecutablePathFinder import CondaExecutablePathFinder
from penvy.conda.CondaPathsResolver import CondaPathsResolver
from penvy.logger.LoggerLevelResolver import LoggerLevelResolver
from penvy.parameters.ParametersResolverInterface import ParametersResolverInterface
from penvy.container.dicontainer import Container
from penvy.poetry.PoetryPathResolver import PoetryPathResolver
from penvy.python.PythonExecutablePathResolver import PythonExecutablePathResolver
from penvy.shell.ShellResolver import ShellResolver


class PenvyConfig(EnvConfig):
    def get_parameters(self) -> dict:
        return {
            "logger": {"name": "env-init"},
            "conda": {"minimal_version": "4.7.12"},
            "git": {"minimal_version": "2.24.0"},
            "poetry": {"install_version": "1.0.0"},
        }

    def get_parameters_resolvers(self, default_config: dict) -> List[ParametersResolverInterface]:
        return [
            ShellResolver(),
            LoggerLevelResolver(),
            CondaPathsResolver(CondaExecutablePathFinder()),
            PythonExecutablePathResolver(),
            PoetryPathResolver(),
        ]

    def get_checks(self, container: Container):
        return [
            container.get_shell_check(),
            container.get_pyproject_checker(),
            container.get_conda_version_check(),
            container.get_git_version_check(),
        ]

    def get_setup_steps(self, container: Container):
        return [
            container.get_conda_environment_creator(),
            container.get_conda_scripts_creator(),
            container.get_bash_profile_creator(),
            container.get_pyfony_env_creator(),
            container.get_file_creator_bashrc(),
            container.get_source_conda_appender_bashrc(),
            container.get_source_pyfony_env_appender_bashrc(),
            container.get_file_creator_zshrc(),
            container.get_source_conda_appender_zshrc(),
            container.get_source_pyfony_env_appender_zshrc(),
            container.get_poetry_installer(),
            container.get_dot_env_creator(),
            container.get_dependencies_installer(),
            container.get_post_merge_hook_creator(),
            container.get_dependencies_install_hook_creator(),
        ]

    def get_tear_down_steps(self, container: Container):
        return [container.get_activate_conda_env_call()]



class ParametersMerger:
    def merge(self, a, b, overwrite=True, path=None):
        "merges b into a"
        if path is None:
            path = []

        for key in b:
            if key in a:
                if isinstance(a[key], dict) and isinstance(b[key], dict):
                    self.merge(a[key], b[key], overwrite, path + [str(key)])
                elif a[key] == b[key]:
                    pass  # same leaf value
                elif overwrite is True:
                    a[key] = b[key]
                else:
                    raise Exception("Conflict at %s" % ".".join(path + [str(key)]))
            else:
                a[key] = b[key]
        return a



from penvy.container.diservice import diservice


class Container:
    def __init__(
        self,
        parameters: dict,
    ):
        self.services = {}
        self._parameters = parameters

    @property
    def parameters(self):
        return self._parameters

    @diservice
    def get_logger(self):
        from penvy.logger.logger_factory import get_logger

        return get_logger(
            self._parameters["logger"]["name"],
            self._parameters["logger"]["level"],
        )

    @diservice
    def get_poetry_installer(self):
        from penvy.poetry.PoetryInstaller import PoetryInstaller

        return PoetryInstaller(
            self._parameters["python"]["executable_path"],
            self._parameters["poetry"]["executable_path"],
            self._parameters["poetry"]["install_version"],
            self.get_logger(),
        )

    @diservice
    def get_conda_environment_creator(self):
        from penvy.conda.CondaEnvironmentCreator import CondaEnvironmentCreator

        return CondaEnvironmentCreator(
            self._parameters["conda"]["executable_path"], self._parameters["project"]["venv_dir"], self.get_logger()
        )

    @diservice
    def get_conda_scripts_creator(self):
        from penvy.conda.CondaScriptsCreator import CondaScriptsCreator

        return CondaScriptsCreator(self._parameters["project"]["venv_dir"], self.get_logger())

    @diservice
    def get_bash_profile_creator(self):
        from penvy.bash.BashProfileCreator import BashProfileCreator

        return BashProfileCreator(
            self._parameters["shell"]["is_bash"],
            self.get_logger(),
        )

    @diservice
    def get_file_creator_bashrc(self):
        import os
        from penvy.shell.RcFileCreator import RcFileCreator

        return RcFileCreator(os.path.expanduser("~") + os.sep + ".bashrc", self._parameters["shell"]["is_bash"], self.get_logger())

    @diservice
    def get_file_creator_zshrc(self):
        import os
        from penvy.shell.RcFileCreator import RcFileCreator

        return RcFileCreator(os.path.expanduser("~") + os.sep + ".zshrc", self._parameters["shell"]["is_zsh"], self.get_logger())

    @diservice
    def get_source_conda_appender_bashrc(self):
        import os
        from penvy.conda.SourceCondaAppender import SourceCondaAppender

        return SourceCondaAppender(
            os.path.expanduser("~") + os.sep + ".bashrc",
            self._parameters["conda"]["conda_sh_path"],
            self._parameters["shell"]["is_bash"],
            self.get_logger(),
        )

    @diservice
    def get_source_conda_appender_zshrc(self):
        import os
        from penvy.conda.SourceCondaAppender import SourceCondaAppender

        return SourceCondaAppender(
            os.path.expanduser("~") + os.sep + ".zshrc",
            self._parameters["conda"]["conda_sh_path"],
            self._parameters["shell"]["is_zsh"],
            self.get_logger(),
        )

    @diservice
    def get_source_pyfony_env_appender_bashrc(self):
        import os
        from penvy.shell.SourcePyfonyEnvAppender import SourcePyfonyEnvAppender

        return SourcePyfonyEnvAppender(
            os.path.expanduser("~") + os.sep + ".bashrc", self._parameters["shell"]["is_bash"], self.get_logger()
        )

    @diservice
    def get_source_pyfony_env_appender_zshrc(self):
        import os
        from penvy.shell.SourcePyfonyEnvAppender import SourcePyfonyEnvAppender

        return SourcePyfonyEnvAppender(os.path.expanduser("~") + os.sep + ".zshrc", self._parameters["shell"]["is_zsh"], self.get_logger())

    @diservice
    def get_pyfony_env_creator(self):
        from penvy.shell.PyfonyEnvCreator import PyfonyEnvCreator

        return PyfonyEnvCreator(self.get_logger())

    @diservice
    def get_dot_env_creator(self):
        from penvy.dotenv.DotEnvCreator import DotEnvCreator

        return DotEnvCreator(self._parameters["project"]["dir"], self.get_logger())

    @diservice
    def get_dependencies_installer(self):
        from penvy.poetry.DependenciesInstaller import DependenciesInstaller

        return DependenciesInstaller(self._parameters["poetry"]["executable_path"], self.get_logger())

    @diservice
    def get_activate_conda_env_call(self):
        from penvy.conda.ActivateCondaEnvCall import ActivateCondaEnvCall

        return ActivateCondaEnvCall(self._parameters["project"]["venv_dir"], self.get_logger())

    @diservice
    def get_shell_check(self):
        from penvy.shell.ShellCheck import ShellCheck

        return ShellCheck(self.get_logger())

    @diservice
    def get_pyproject_checker(self):
        from penvy.poetry.PyprojectChecker import PyprojectChecker

        return PyprojectChecker(self.get_logger())

    @diservice
    def get_conda_version_check(self):
        from penvy.conda.CondaVersionCheck import CondaVersionCheck

        return CondaVersionCheck(
            self._parameters["conda"]["minimal_version"],
            self._parameters["conda"]["executable_path"],
            self.get_logger(),
        )

    @diservice
    def get_git_version_check(self):
        from penvy.git.GitVersionCheck import GitVersionCheck

        return GitVersionCheck(
            self._parameters["git"]["minimal_version"],
            self.get_logger(),
        )

    @diservice
    def get_post_merge_hook_creator(self):
        from penvy.git.PostMergeHookCreator import PostMergeHookCreator

        return PostMergeHookCreator(self._parameters["project"]["dir"] + "/.git/hooks/post-merge", self.get_logger())

    @diservice
    def get_dependencies_install_hook_creator(self):
        from penvy.poetry.DependenciesInstallHookCreator import DependenciesInstallHookCreator

        return DependenciesInstallHookCreator(
            self._parameters["project"]["dir"] + "/.git/hooks/post-merge",
            self._parameters["poetry"]["executable_path"],
            self.get_logger(),
        )



import os
from logging import Logger
from penvy.setup.SetupStepInterface import SetupStepInterface
from penvy.shell.home_path_shortener import shorten_home_path


class RcFileCreator(SetupStepInterface):
    def __init__(
        self,
        rc_file_path: str,
        expected_shell_is_running: callable,
        logger: Logger,
    ):
        self._rc_file_path = rc_file_path
        self._rc_file_path_shorten = shorten_home_path(rc_file_path)
        self._expected_shell_is_running = expected_shell_is_running
        self._logger = logger

    def get_description(self):
        return f"Create {self._rc_file_path_shorten}"

    def run(self):
        self._logger.info(f"Creating {self._rc_file_path_shorten}")

        open(self._rc_file_path, "a").close()

    def should_be_run(self) -> bool:
        return self._expected_shell_is_running and not os.path.isfile(self._rc_file_path)



from abc import ABC, abstractmethod
from argparse import Namespace


class ParametersResolverInterface(ABC):
    @abstractmethod
    def resolve(self, config: dict, cli_args: Namespace):
        pass



from argparse import ArgumentTypeError


def str2bool(v):
    if isinstance(v, bool):
        return v

    if v.lower() in ("yes", "true", "t", "y", "1"):
        return True

    if v.lower() in ("no", "false", "f", "n", "0"):
        return False

    raise ArgumentTypeError("Boolean value expected")
